/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#include "../include/rfs.h"
#include "../common/rf_common.h"

#if RF_OPT_HAVE_CODE_PAGE
#include "unicode.h"

rf_int_t g_path_code_page = RF_OPT_DEF_CODE_PAGE;

rf_int_t rf_set_codepage(rf_int_t cp)
{
    g_path_code_page = cp;
    return 0;
}

rf_wchar_t w2A(rf_wchar_t chr)
{
    rf_int_t l = mb_bytes();
    if (l == 1) {
        chr = w2a(chr);
        if (chr)
            chr = sb_upper_ex(chr - 0x80);
        return chr;
    }
    return w2a(wtoupper(chr));
}

rf_wchar_t e2A(rf_wchar_t chr)
{
    rf_int_t l = mb_bytes();
    if (l == 1 && chr >= 0x80 && chr < 0x100)
        chr = sb_upper_ex(chr - 0x80);
    return chr;
}

rf_char_t* wchar_to_utf8(rf_u32_t wc, rf_char_t* output, rf_int_t outsize, rf_int_t* outlen)
{
    if (wc <= 0x7f) {
        if (outsize < 1)
            return NULL;
        *output++ = (char) wc;
        *outlen = 1;
    } else if (wc <= 0x7ff) {
        if (outsize < 2)
            return NULL;
        *output++ = 0xc0 | (wc >> 6);
        *output++ = 0x80 | (wc & 0x3f);
        *outlen = 2;
    } else if (wc <= 0xffff) {
        if (outsize < 3)
            return NULL;
        *output++ = 0xe0 | (wc >> 12);
        *output++ = 0x80 | ((wc >> 6) & 0x3f);
        *output++ = 0x80 | (wc & 0x3f);
        *outlen = 3;
    } else if (wc <= 0x1fffff) {
        if (outsize < 4)
            return NULL;
        *output++ = 0xf0 | (wc >> 18);
        *output++ = 0x80 | ((wc >> 12) & 0x3f);
        *output++ = 0x80 | ((wc >> 6) & 0x3f);
        *output++ = 0x80 | (wc & 0x3f);
        *outlen = 4;
    } else if (wc <= 0x3ffffff) {
        if (outsize < 5)
            return NULL;
        *output++ = 0xf8 | (wc >> 24);
        *output++ = 0x80 | ((wc >> 18) & 0x3f);
        *output++ = 0x80 | ((wc >> 12) & 0x3f);
        *output++ = 0x80 | ((wc >> 6) & 0x3f);
        *output++ = 0x80 | (wc & 0x3f);
        *outlen = 5;
    } else if (wc <= 0x7fffffff) {
        if (outsize < 6)
            return NULL;
        *output++ = 0xfc | (wc >> 30);
        *output++ = 0x80 | ((wc >> 24) & 0x3f);
        *output++ = 0x80 | ((wc >> 18) & 0x3f);
        *output++ = 0x80 | ((wc >> 12) & 0x3f);
        *output++ = 0x80 | ((wc >> 6) & 0x3f);
        *output++ = 0x80 | (wc & 0x3f);
        *outlen = 6;
    } else {
        return NULL;
    }
    return output;
}

rf_int_t str2cp(rf_char_t* cp)
{
    if (cp == NULL)
        return g_path_code_page;
    if (!strcmp(cp, "utf8"))
        return RF_CODE_PAGE_UTF8;

#if RF_OPT_CODE_PAGE_437
    if (!strcmp(cp, "437"))
        return RF_CODE_PAGE_437;
#endif

#if RF_OPT_CODE_PAGE_720
    if (!strcmp(cp, "720"))
        return RF_CODE_PAGE_720;
#endif

#if RF_OPT_CODE_PAGE_737
    if (!strcmp(cp, "737"))
        return RF_CODE_PAGE_737;
#endif

#if RF_OPT_CODE_PAGE_775
    if (!strcmp(cp, "755"))
        return RF_CODE_PAGE_775;
#endif

#if RF_OPT_CODE_PAGE_850
    if (!strcmp(cp, "850"))
        return RF_CODE_PAGE_850;
#endif

#if RF_OPT_CODE_PAGE_852
    if (!strcmp(cp, "852"))
        return RF_CODE_PAGE_852;
#endif

#if RF_OPT_CODE_PAGE_855
    if (!strcmp(cp, "855"))
        return RF_CODE_PAGE_855;
#endif

#if RF_OPT_CODE_PAGE_857
    if (!strcmp(cp, "857"))
        return RF_CODE_PAGE_857;
#endif

#if RF_OPT_CODE_PAGE_862
    if (!strcmp(cp, "862"))
        return RF_CODE_PAGE_862;
#endif

#if RF_OPT_CODE_PAGE_866
    if (!strcmp(cp, "866"))
        return RF_CODE_PAGE_866;
#endif

#if RF_OPT_CODE_PAGE_932
    if (!strcmp(cp, "932"))
        return RF_CODE_PAGE_932;
#endif

#if RF_OPT_CODE_PAGE_936
    if (!strcmp(cp, "936"))
        return RF_CODE_PAGE_936;
#endif

#if RF_OPT_CODE_PAGE_949
    if (!strcmp(cp, "949"))
        return RF_CODE_PAGE_949;
#endif

#if RF_OPT_CODE_PAGE_950
    if (!strcmp(cp, "950"))
        return RF_CODE_PAGE_950;
#endif

    return -1;
}

#endif
