/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#include "../include/rfs.h"
#ifdef RF_COMPILER_IS_MSVC
#include <Windows.h>
#include <assert.h>
#endif

#ifdef RF_COMPILER_IS_MSVC
static rf_int_t utf8_mb(const rf_void_t* in_str, rf_void_t* out_str, rf_int_t out_len)
{
    rf_int_t ret;
    rf_u8_t buf[RF_MAX_FNAME];

    ret = MultiByteToWideChar(CP_UTF8, 0, in_str, -1, (LPWSTR)buf, sizeof(buf) / 2);
    if (!ret)
        return 0;
    ret = WideCharToMultiByte(CP_ACP, 0, (LPWSTR)buf, -1, out_str, out_len, NULL, NULL);
    return ret;
}

rf_int_t rf_iconv(const rf_void_t* in_str, rf_int_t in_cp, rf_void_t* out_str, rf_int_t out_len, rf_int_t out_cp)
{
    rf_size_t in_len;
    rf_bool_t ret = FALSE;

    if (in_cp == RF_CODE_PAGE_UCS2B || in_cp == RF_CODE_PAGE_UCS2L)
        in_len = wcslen(in_str) * 2;
    else
        in_len = strlen(in_str);

    if (in_cp == out_cp) {
        if (in_cp == RF_CODE_PAGE_UCS2B || in_cp == RF_CODE_PAGE_UCS2L) {
            assert(in_len + 2 <= out_len);
            wcscpy_s(out_str, out_len / 2, in_str);
            return 0;
        } else {
            assert(in_len + 1 <= out_len);
            strcpy_s(out_str, out_len, in_str);
            return 0;
        }
    }

    if (in_cp == RF_CODE_PAGE_UCS2L && out_cp == RF_CODE_PAGE_936)
        ret = (rf_bool_t)WideCharToMultiByte(CP_ACP, 0, (LPCWSTR)in_str, -1, out_str, out_len, NULL, NULL);
    else if (in_cp == RF_CODE_PAGE_936 && out_cp == RF_CODE_PAGE_UCS2L)
        ret = (rf_bool_t)MultiByteToWideChar(CP_ACP, 0, in_str, -1, (LPWSTR)out_str, out_len);
    else if (in_cp == RF_CODE_PAGE_UTF8 && out_cp == RF_CODE_PAGE_936)
        ret = (rf_bool_t)utf8_mb(in_str, (LPWSTR)out_str, out_len);
    else
        return RF_EINVAL;
    return 0;
}
#else
rf_int_t rf_iconv(const rf_void_t* in_str, rf_int_t in_cp, rf_void_t* out_str, rf_int_t out_len, rf_int_t out_cp)
{
    return 1;
}
#endif
