/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#ifndef __RF_DEBUG_H__
#define __RF_DEBUG_H__

#define RF_DBG_LEVEL_DBG     0x00
#define RF_DBG_LEVEL_WARNING 0x01
#define RF_DBG_LEVEL_ERROR   0x02
#define RF_DBG_LEVEL_BUG     0x03
#define RF_DBG_MASK_LEVEL    0x03

#define RF_DBG_ON            0x80
#define RF_DBG_OFF           0x00
#define RF_DBG_MIN_LEVEL     0x00

#ifdef RF_DEBUG
#   ifndef RF_PLATFORM_ASSERT
#   error "If you want to use RF_ASSERT, RF_PLATFORM_ASSERT(message) needs to be defined in your rf_arch.h"
#   endif
#   ifndef RF_PLATFORM_PRINTF
#   error "If you want to use RF_DEBUG, RF_PLATFORM_PRINTF(message) needs to be defined in your rf_arch.h"
#   endif
#   define RF_ASSERTE(message, assertion) do { if (!(assertion)) { RF_PLATFORM_ASSERT(message); }} while(0)
#   define RF_ASSERT_STATIC(x) do { typedef int __rf_static_assert__[(x)? 1 : -1];  __rf_static_assert__ __a; __a[0] = 0; } while(0)
#   define RF_PRINTF(debug, message) do { \
                               if ( ( (rf_u16_t)((debug) & RF_DBG_MASK_LEVEL) >= RF_DBG_MIN_LEVEL)) { \
                                 RF_PLATFORM_PRINTF(message); \
                               } \
                             } while(0)
#   define RF_PLATFORM_ERROR(message) RF_PLATFORM_ASSERT(message)
#else
#   define RF_ASSERTE(message, assertion)
#   define RF_ASSERT_STATIC(x)
#   define RF_PRINTF(debug, message)
#   define RF_PLATFORM_ERROR(message)
#endif

#define RF_PERROR(message, expression, handler) do { if (expression) { \
  RF_PLATFORM_ERROR(message); handler;}} while(0)

#endif

