/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#include "../include/rfs.h"
#include "rf_common.h"

rf_int_t g_fs_last_errno;
rf_int_t* rf_get_errno(rf_void_t)
{
    return &g_fs_last_errno;
}

#if RF_OPT_USE_STRERROR
rf_char_t* rf_strerror(rf_int_t errnum)
{
    switch (errnum) {
    case RF_ESUCCESS:
        return "Execute successfully";
    case RF_ERROR:
        return "Unknown error";
    case RF_EINVAL:
        return "Invalid argument";
    case RF_EBADF:
        return "Bad file number";
    case RF_ENOMEM:
        return "Out of memory";
    case RF_EBUFSMALL:
        return "Input buffer is too small";
    case RF_EACCES:
        return "Permission denied";
    case RF_EPERM:
        return "Operation not permitted";
    case RF_ENAMETOOLONG:
        return "File name is too long";
    case RF_ENAMEENCODING:
        return "File name encoding error";
    case RF_EISDIR:
        return "Is a directory";
    case RF_ENOTDIR:
        return "Not a directory";
    case RF_EMFILE:
        return "Too many open files";
    case RF_ENOENT:
        return "No such file or directory";
    case RF_ENDFILE:
        return "end of file";
    case RF_ENOSPC:
        return "No space left on device";
    case RF_EFBIG:
        return "File too large";
    case RF_EXDEV:
        return "Cross-device link";
    case RF_ENOTEMPTY:
        return "Directory not empty";

    case RF_EMVOL:
        return "Too many open volume";
    case RF_EIO:
        return "Input/output error";
    case RF_ENOFS:
        return "Invalid file system";
    case RF_EWRPRT:
        return "Write Protected error";
    case RF_EROFS:
        return "Read-only file system";
    default:
        return "Unknown Error code";
    }
}
#endif
