/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#ifndef __RF_MACRO_H__
#define __RF_MACRO_H__

#ifndef min
#define min(a,b)            (((a) < (b)) ? (a) : (b))
#endif

#ifndef max
#define max(a,b)            (((a) > (b)) ? (a) : (b))
#endif

#ifndef LOWORD
#define LOWORD(l)           ((rf_u16_t)((rf_u32_t)(l) & 0xffff))
#endif

#ifndef HIWORD
#define HIWORD(l)           ((rf_u16_t)((rf_u32_t)(l) >> 16))
#endif

#ifndef LOBYTE
#define LOBYTE(w)           ((rf_u8_t)((rf_u32_t)(w) & 0xff))
#endif

#ifndef HIBYTE
#define HIBYTE(w)           ((rf_u8_t)((rf_u32_t)(w) >> 8))
#endif

#define align_size_u32(x, b)   (((rf_u32_t)(x) + ((rf_u32_t)(b) - 1)) & ~((rf_u32_t)(b) - 1))
#define RF_CONTAINING_RECORD(addr,type,field) (void *)((type*)((rf_char_t*)addr - (rf_size_t)&((type*)0)->field))

#define u8_read(buf,idx)        ((buf)[(idx)])
#define u8_write(buf,idx, d)    ((buf)[(idx)]=(d))
#if RF_OPT_CPU_BIG_ENDIAN ==0
#define u16_read(ptr,idx)       (rf_u16_t)(*(rf_u16_t*)((rf_u8_t*)(ptr)+idx ))
#define u32_read(ptr,idx)       (rf_u32_t)(*(rf_u32_t*)((rf_u8_t*)(ptr)+idx ))
#define u64_read(ptr,idx)       (rf_u64_t)(*(rf_u64_t*)((rf_u8_t*)(ptr)+idx ))
#define u16_write(ptr,idx,val)  *(rf_u16_t*)((rf_u8_t*)(ptr)+idx )=(rf_u16_t)(val)
#define u32_write(ptr,idx,val)  *(rf_u32_t*)((rf_u8_t*)(ptr)+idx )=(rf_u32_t)(val)
#define u64_write(ptr,idx,val)  *(rf_u64_t*)((rf_u8_t*)(ptr)+idx )=(rf_u64_t)(val)
#else
#define u16_read(ptr,idx)       (rf_u16_t)(((rf_u16_t)*((rf_u8_t*)(ptr)+idx+1)<<8)|(rf_u16_t)*((rf_u8_t*)(ptr)+idx))
#define u32_read(ptr,idx)       (rf_u32_t)(((rf_u32_t)*((rf_u8_t*)(ptr)+idx+3)<<24)|((rf_u32_t)*((rf_u8_t*)(ptr)+idx+2)<<16)|((rf_u16_t)*((rf_u8_t*)(ptr)+idx+1)<<8)|*((rf_u8_t*)(ptr)+idx))
#define u16_write(ptr,idx,val)  do {*((rf_u8_t*)(ptr)+idx)=(rf_u8_t)(val); *((rf_u8_t*)(ptr)+idx+1)=(rf_u8_t)((rf_u16_t)(val)>>8); } while( 0 )
#define u32_write(ptr,idx,val)  do {*((rf_u8_t*)(ptr)+idx)=(rf_u8_t)(val); *((rf_u8_t*)(ptr)+idx+1)=(rf_u8_t)((rf_u16_t)(val)>>8); *((rf_u8_t*)(ptr)+idx+2)=(rf_u8_t)((rf_u32_t)(val)>>16); *((rf_u8_t*)(ptr)+idx+3)=(rf_u8_t)((rf_u32_t)(val)>>24); } while( 0 )
//#define   u64_write(ptr,idx,val)  do {*((rf_u8_t*)(ptr)+idx)=(rf_u8_t)(val); *((rf_u8_t*)(ptr)+idx+1)=(rf_u8_t)((rf_u16_t)(val)>>8); *((rf_u8_t*)(ptr)+idx+2)=(rf_u8_t)((rf_u32_t)(val)>>16); *((rf_u8_t*)(ptr)+idx+3)=(rf_u8_t)((rf_u32_t)(val)>>24); } while( 0 )
#endif

#define is_absolute_path(path)  ((path[0]=='\\'||path[0]=='/'))
#define NULL_CLUSTER            0xffffffff
#define INVOID_SECTOR_SIZE(s)   (s != 512 && s != 1024 && s != 2048 && s != 4096)

#define __rf_mstring__(x)                                   #x
#define __rf_mstring_ex__(x)                                __rf_mstring__(x)

#define __rf_mconcat__(a, b)                                a##b
#define __rf_mconcat_ex__(a, b)                             __rf_mconcat__(a, b)

#define __rf_mconcat3__(a, b, c)                            a##b##c
#define __rf_mconcat3_ex__(a, b, c)                         __rf_mconcat3__(a, b, c)

#define __rf_mconcat4__(a, b, c, d)                         a##b##c##d
#define __rf_mconcat4_ex__(a, b, c, d)                      __rf_mconcat4__(a, b, c, d)

#define __rf_mconcat5__(a, b, c, d, e)                      a##b##c##d##e
#define __rf_mconcat5_ex__(a, b, c, d, e)                   __rf_mconcat5__(a, b, c, d, e)

#define __rf_mconcat6__(a, b, c, d, e, f)                   a##b##c##d##e##f
#define __rf_mconcat6_ex__(a, b, c, d, e, f)                __rf_mconcat6__(a, b, c, d, e, f)

#define __rf_mconcat7__(a, b, c, d, e, f, g)                a##b##c##d##e##f##g
#define __rf_mconcat7_ex__(a, b, c, d, e, f, g)             __rf_mconcat7__(a, b, c, d, e, f, g)

#define __rf_mconcat8__(a, b, c, d, e, f, g, h)             a##b##c##d##e##f##g##h
#define __rf_mconcat8_ex__(a, b, c, d, e, f, g, h)          __rf_mconcat8__(a, b, c, d, e, f, g, h)

#define __rf_mconcat9__(a, b, c, d, e, f, g, h, i)          a##b##c##d##e##f##g##h##i
#define __rf_mconcat9_ex__(a, b, c, d, e, f, g, h, i)       __rf_mconcat9__(a, b, c, d, e, f, g, h, i)

#define __rf_mstrcat__(a, b)                                a b
#define __rf_mstrcat3__(a, b, c)                            a b c
#define __rf_mstrcat4__(a, b, c, d)                         a b c d
#define __rf_mstrcat5__(a, b, c, d, e)                      a b c d e
#define __rf_mstrcat6__(a, b, c, d, e, f)                   a b c d e f
#define __rf_mstrcat7__(a, b, c, d, e, f, g)                a b c d e f g
#define __rf_mstrcat8__(a, b, c, d, e, f, g, h)             a b c d e f g h
#define __rf_mstrcat9__(a, b, c, d, e, f, g, h, i)          a b c d e f g h i

#endif
