/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#include "../include/rfs.h"
#include "rf_common.h"

rf_tchar_t path_get_root(const rf_tchar_t* path)
{
    if (path[0] == '\0' || path[1] == '\0')
        return '\0';
    if (path[1] == ':')
        return path[0];
    return '\0';
}

const rf_char_t* path_get_root_dir(const rf_char_t* path)
{
    if (path[1] == ':')
        return path + 2;
    return path;
}

#ifndef RF_OPT_PC_PLAT
rf_bool_t path_remove_filespec(rf_char_t* pszPath)
{
    rf_int_t i;
    for (i = (rf_int_t)strlen(pszPath); i >= 0; i--) {
        if (pszPath[i] == '/' || pszPath[i] == '\\') {
            pszPath[i + 1] = '\0';
            return TRUE;
        }
    }
    return FALSE;
}

const rf_char_t* path_get_filespec(const rf_char_t*  pszPath)
{
    rf_int_t i;
    for (i = (rf_int_t)strlen(pszPath); i >= 0; i--) {
        if (pszPath[i] == '/' || pszPath[i] == '\\')
            return pszPath + i + 1;
    }
    return pszPath;
}
#endif

/* жϵǰǷһĿ¼ */
#if RF_OPT_USE_NTFS || RF_OPT_USE_EXT2
rf_int_t path_is_last(const rf_tchar_t** path, rf_int_t* last_path)
{
    const rf_tchar_t* p;
    rf_tchar_t w;
    rf_int_t offset = 0;

    for (p = *path; *p == '/' || *p == '\\'; p++);
    if (p[0] == '\0') {
        *last_path = 1;
        return RF_ESUCCESS;
    }

    /* Ҫ ǿո͵ַ ΪһĿ¼ */
    for (;;) {
        w = p[offset++];
        if (w == '\0')
            break;
        if (w != ' ' && w != '.')
            return RF_ESUCCESS;
    }
    *last_path = 1;
    return RF_ESUCCESS;
}
#endif

#if RF_OPT_USE_GETCWD
/* · תΪ· */
rf_int_t path_full(rf_char_t* work_dir, rf_int_t max_len, const rf_char_t* path)
{
    rf_char_t* dst0, *dst, *src;
    rf_char_t ret, c;

    if (path[0] == '/' || path[0] == '\\') {
        /* ǸĿ¼ · */
        ret = rf_strcpy_s(work_dir, max_len, path);
    } else if (path[0] != '\0' && path[1] == ':') {
        /*  ̾· */
        ret = rf_strcpy_s(work_dir, max_len, path + 2);
    } else {
        /* ·β */
        if (work_dir[1] != '\0') {
            ret = rf_strcat_s(work_dir, max_len, RF_PATH_DEF_SEP_STR);
            if (ret)
                return ret;
        }
        ret = rf_strcat_s(work_dir, max_len, path);
    }
    if (ret)
        return ret;

    rf_str_replace_all(work_dir, '/', RF_PATH_DEF_SEP_CHR);

    src = work_dir;
    dst = work_dir;
    dst0 = dst;
    while (1) {
        c = *src;
        if (c == '.') {
            if (!src[1])
                src ++; /* '.' and ends */
            else if (src[1] == RF_PATH_DEF_SEP_CHR) {
                /* './' case */
                src += 2;
                while ((*src == RF_PATH_DEF_SEP_CHR) && (*src != '\0'))
                    src ++;
                continue;
            } else if (src[1] == '.') {
                if (!src[2]) {
                    /* '..' and ends case */
                    src += 2;
                    goto up_one;
                } else if (src[2] == RF_PATH_DEF_SEP_CHR) {
                    /* '../' case */
                    src += 3;

                    while ((*src == RF_PATH_DEF_SEP_CHR) && (*src != '\0'))
                        src ++;
                    goto up_one;
                }
            }
        }

        /* copy up the next '/' and erase all '/' */
        while ((c = *src++) != '\0' && c != RF_PATH_DEF_SEP_CHR)
            * dst ++ = c;

        if (c == RF_PATH_DEF_SEP_CHR) {
            *dst ++ = RF_PATH_DEF_SEP_CHR;
            while (c == RF_PATH_DEF_SEP_CHR)
                c = *src++;
            src --;
        } else if (!c)
            break;
        continue;

up_one:
        dst --;
        if (dst < dst0) {
            /* չʧ */
            work_dir[0] = RF_PATH_DEF_SEP_CHR;
            work_dir[1] = '\0';
            RF_PERROR("error: invalid path\n", 1, return RF_EINVAL);
        }
        while (dst0 < dst && dst[-1] != RF_PATH_DEF_SEP_CHR)
            dst --;
    }

    *dst = '\0';
    /* remove '/' in the end of path if exist */
    dst --;
    if ((dst != work_dir) && (*dst == RF_PATH_DEF_SEP_CHR))
        *dst = '\0';
    /* Ĭϴ Ŀ¼ */
    if (work_dir[0] == '\0') {
        work_dir[0] = RF_PATH_DEF_SEP_CHR;
        work_dir[1] = '\0';
    }
    return 0;
}
#endif
