/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#ifndef __RF_PATH_H__
#define __RF_PATH_H__

#define NS          11      /* Index of name status byte in fn[] */
#define NS_LOSS     0x01    /* Out of 8.3 format */
#define NS_LFN      0x02    /* Force to create LFN entry */
#define NS_LAST     0x04    /* Last segment */
#define NS_BODY     0x08    /* Lower case flag (body) */
#define NS_EXT      0x10    /* Lower case flag (ext) */
#define NS_DOT      0x20    /* Dot entry */
#define DDE         0xE5    /* Deleted directory entry mark in DIR_Name[0] */
#define NDDE        0x05    /* Replacement of the character collides with DDE */

rf_tchar_t path_get_root(const rf_tchar_t* path);
const rf_char_t* path_get_root_dir(const rf_char_t* path);
rf_bool_t path_remove_filespec(rf_char_t* pszPath);
const rf_char_t* path_get_filespec(const rf_char_t*  pszPath);

rf_int_t path_is_last(const rf_tchar_t** path, rf_int_t* last_path);
rf_int_t path_get_sfn_lfn(const rf_tchar_t** path, rf_char_t* sfn, rf_wchar_t* lfn);
rf_int_t path_get_fn(const rf_char_t** path, rf_char_t* name, rf_int_t len, rf_int_t cp);

rf_int_t path_full(rf_char_t* work_dir, rf_int_t max_len, const rf_char_t* path);

rf_void_t fat_gen_numname(rf_char_t* dst, const rf_char_t* src, const rf_wchar_t* lfn, rf_u32_t seq);
rf_void_t fat_convert_raw_name(rf_char_t* raw_name, rf_int_t nt, rf_char_t* file_name);
rf_int_t fat_convert_short_name(const rf_char_t* name, rf_char_t* sfn);

#endif
