/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#ifndef __EXT2_FILE_H__
#define __EXT2_FILE_H__

#define     TYPE_DIRECT_BLOCK               0
#define     TYPE_INDIRECT_BLOCK             1
#define     TYPE_DOUBLE_INDIRECT_BLOCK      2
#define     TYPE_TRIPLE_INDIRECT_BLOCK      3

struct _block_offset_t {
    rf_u8_t type;
    union {
        struct {
            rf_u16_t offset;
        } direct;
        struct {
            rf_lloff_t  sector;
            rf_u16_t    offset;
        } indirect;
        struct {
            rf_lloff_t  indirect_sector;
            rf_lloff_t  direct_sector;
            rf_u16_t    indirect_offset;
            rf_u16_t    direct_offset;
        } double_indirect;
        struct {
            rf_lloff_t  double_sector;
            rf_lloff_t  indirect_sector;
            rf_lloff_t  direct_sector;
            rf_u16_t    double_offset;
            rf_u16_t    indirect_offset;
            rf_u16_t    direct_offset;
        } triple_indirect;
    } u;
};
typedef struct _block_offset_t block_offset_t, *block_offset_ref_t;

struct _rf_ext2_file_t {
    /* ļ ж */
    rf_u8_t             fs_type: 4;
    /* 򿪻߹ر ־λ */
    rf_u8_t             open_flag: 1;
    /* 0ֻ 1д 2дȨ */
    rf_u8_t             access_flag: 2;
    /* 1 ļ־ */
    rf_u8_t             sector_buf_flag: 1;
    /* 1 ǰΪĿ¼ */
    rf_u8_t             root_dir_flag: 1;
    /* Ŀ¼ڷ */
    rf_ext2_volume_ref_t vol;
#if RF_OPT_USE_FILE_CTX
    /* ļ */
    rf_void_ref_t       ctx;
#endif
    /* ̬malloc fileʱ ʹ */
#if (RF_OPT_USE_MALLOC == 1)
    list_t list;
#endif

    /* дǷѼ ־λ */
    rf_u8_t             gather_cluester_bit: 1;
    /* ļȸı־λ */
#if ( RF_OPT_FS_READONLY == 0 )
    rf_u8_t             attrib_change_bit: 1;
#endif

    /* ļ */
    rf_u32_t            attr;
    /* ļ */
    rf_u64_t            length;

    /* ȡƫ */
    rf_u32_t            blocks_offset;
    /* 32λΪ 32λΪڴƫ */
    rf_fentry_t     entry;
    /* ǰļ ϼĿ¼ ʼ*/
    rf_u32_t            updir_entry;
    /* ļϼĿ¼ƫ */
    rf_u32_t            up_dir_offset;

    /* ϴѼһ */
    rf_u32_t            last_cluster;
    /* һ  */
    rf_lloff_t          last_cache_sector;
    /* ȡʱ ʹõƫ */
    block_offset_t      block;

    /* ļĿ¼ǰƫ */
    rf_u64_t            current_offset;
    /* ļؽṹ */
    rf_chain_idx_t      clust;
};
typedef struct _rf_ext2_file_t rf_ext2_file_t, *rf_ext2_file_ref_t;

rf_int_t ext2_file_open(rf_ext2_file_ref_t file, const rf_char_t* name, rf_int_t flags);
rf_int_t ext2_get_inode_entry(rf_ext2_volume_ref_t vol, rf_u32_t inum, rf_fentry_ref_t entry);
rf_int_t ext2_file_raw_open(rf_ext2_file_ref_t file, rf_fentry_t* fentry, rf_u32_t open_flag);
rf_int_t ext2_file_read(rf_ext2_file_ref_t file, rf_void_t* buf, rf_u32_t len, rf_u32_t* plen);
rf_int_t ext2_file_close(rf_ext2_file_ref_t file);
rf_int_t ext2_file_get_stat(rf_ext2_file_ref_t file, rf_stat_ref_t st);
rf_int_t ext2_file_get_ftime(rf_ext2_file_ref_t file, rf_file_ftime_ref_t pt);
rf_int_t ext2_file_read_dir(rf_ext2_file_ref_t file, const rf_char_t* pattern, rf_dirent_ref_t pfd);
rf_int_t ext2_file_lseek(rf_ext2_file_ref_t file, rf_u64_t offset, rf_int_t origin);

rf_bool_t ext2_check_file_access(rf_ext2_file_ref_t file);

#endif
