/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#ifndef __FAT_API_H__
#define __FAT_API_H__

rf_vol_comm_ref_t fat_vol_alloc(rf_void_t);
rf_void_t fat_vol_free(rf_vol_comm_ref_t vol);

rf_int_t fat_get_volume_handle_count(rf_int_t* available);
rf_int_t fat_get_file_handle_count(rf_int_t* available);
rf_void_t fat_init(rf_void_t);

rf_int_t fat_mkfs(const rf_char_t* devname, const rf_void_t* ctx) reentrant;
rf_int_t fat_mount(struct rf_vol_ctx* vctx, rf_mount_param_t* param) reentrant;
rf_int_t fat_unmount(struct rf_vol_ctx* vctx) reentrant;
rf_int_t fat_ioctl_vol(struct rf_vol_ctx* vctx, rf_int_t cmd, rf_void_t* args) reentrant;
rf_int_t fat_raw_open(struct rf_file_ctx* fd, rf_fentry_t* fentry, rf_int_t flags) reentrant;
rf_int_t fat_open(struct rf_file_ctx* fd, const rf_char_t* file_name, rf_int_t flags) reentrant;
rf_int_t fat_close(struct rf_file_ctx* fd) reentrant;
rf_int_t fat_ioctl(struct rf_file_ctx* fd, rf_int_t cmd, rf_void_t* args) reentrant;
rf_int_t fat_readdir(struct rf_file_ctx* fd, const rf_char_t* pattern, rf_dirent_t* dirp) reentrant;
rf_int_t fat_lseek(struct rf_file_ctx* fd, rf_fsize_t offset, rf_int_t origin, rf_fsize_t* now_offset) reentrant;
rf_int_t fat_read(struct rf_file_ctx* fd, rf_void_t* buf, rf_size_t len, rf_size_t* plen) reentrant;
rf_int_t fat_write(struct rf_file_ctx* fd, const rf_void_t* buf, rf_size_t len, rf_size_t* plen) reentrant;
rf_int_t fat_unlink(struct rf_vol_ctx* vctx, const rf_char_t* pathname) reentrant;
rf_int_t fat_rename(struct rf_vol_ctx* vctx, const rf_char_t* oldname, const rf_char_t* newname) reentrant;

#endif