/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#ifndef __RF_COMPILER_H__
#define __RF_COMPILER_H__

/* intel c++ */
#if defined(__INTEL_COMPILER)
#   define RF_COMPILER_IS_INTEL
#   define RF_COMPILER_VERSION_BT(major, minor)     (__INTEL_COMPILER > ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_BE(major, minor)     (__INTEL_COMPILER >= ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_EQ(major, minor)     (__INTEL_COMPILER == ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_LT(major, minor)     (__INTEL_COMPILER < ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_LE(major, minor)     (__INTEL_COMPILER <= ((major) * 100 + (minor)))
#   define RF_COMPILER_STRING                       "intel c/c++"
#   if (__INTEL_COMPILER == 600)
#       define RF_COMPILER_VERSION_STRING           "intel c/c++ 6.0"
#   elif (__INTEL_COMPILER == 700)
#       define RF_COMPILER_VERSION_STRING           "intel c/c++ 7.0"
#   elif (__INTEL_COMPILER == 800)
#       define RF_COMPILER_VERSION_STRING           "intel c/c++ 8.0"
#   elif (__INTEL_COMPILER == 900)
#       define RF_COMPILER_VERSION_STRING           "intel c/c++ 9.0"
#   elif (__INTEL_COMPILER == 1000)
#       define RF_COMPILER_VERSION_STRING           "intel c/c++ 10.0"
#   elif (__INTEL_COMPILER == 1100)
#       define RF_COMPILER_VERSION_STRING           "intel c/c++ 11.0"
#   elif (__INTEL_COMPILER == 1110)
#       define RF_COMPILER_VERSION_STRING           "intel c/c++ 11.1"
#   elif (__INTEL_COMPILER == 1200)
#       define RF_COMPILER_VERSION_STRING           "intel c/c++ 12.0"
#   elif (__INTEL_COMPILER == 1210)
#       define RF_COMPILER_VERSION_STRING           "intel c/c++ 12.1"
#   elif (__INTEL_COMPILER == 1300)
#       define RF_COMPILER_VERSION_STRING           "intel c/c++ 13.0"
#   elif (__INTEL_COMPILER == 1310)
#       define RF_COMPILER_VERSION_STRING           "intel c/c++ 13.1"
#   elif (__INTEL_COMPILER == 1400)
#       define RF_COMPILER_VERSION_STRING           "intel c/c++ 14.0"
#   elif (__INTEL_COMPILER == 1410)
#       define RF_COMPILER_VERSION_STRING           "intel c/c++ 14.1"
#   elif (__INTEL_COMPILER == 9999)
#       define RF_COMPILER_VERSION_STRING           "intel c/c++ mainline"
#   else
#       define RF_COMPILER_VERSION_STRING            "intel Unknown"
#   endif

/* borland c++ */
#elif defined(__BORLANDC__)
#   define RF_COMPILER_IS_BORLAND
#   define RF_COMPILER_VERSION_BT(major, minor)     (__BORLANDC__ > ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_BE(major, minor)     (__BORLANDC__ >= ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_EQ(major, minor)     (__BORLANDC__ == ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_LT(major, minor)     (__BORLANDC__ < ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_LE(major, minor)     (__BORLANDC__ <= ((major) * 100 + (minor)))
#   define RF_COMPILER_STRING                       "borland c/c++"
#   if 0
#       define RF_COMPILER_VERSION_STRING           "borland c++ 4.52"
#   elif 0
#       define RF_COMPILER_VERSION_STRING           "borland c++ 5.5"
#   elif (__BORLANDC__ == 0x0551)
#       define RF_COMPILER_VERSION_STRING           "borland c++ 5.51"
#   elif (__BORLANDC__ == 0x0560)
#       define RF_COMPILER_VERSION_STRING           "borland c++ 5.6"
#   elif (__BORLANDC__ == 0x0564)
#       define RF_COMPILER_VERSION_STRING           "borland c++ 5.6.4 (c++ builderx)"
#   elif (__BORLANDC__ == 0x0582)
#       define RF_COMPILER_VERSION_STRING           "borland c++ 5.82 (turbo c++)"
#   else
#       define RF_COMPILER_VERSION_STRING           "borland Unknown"
#   endif

/* gnu c/c++ */
#elif defined(__GNUC__)
#   define RF_COMPILER_IS_GCC
#   define RF_COMPILER_VERSION_BT(major, minor)     ((__GNUC__ * 100 + __GNUC_MINOR__) > ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_BE(major, minor)     ((__GNUC__ * 100 + __GNUC_MINOR__) >= ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_EQ(major, minor)     ((__GNUC__ * 100 + __GNUC_MINOR__) == ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_LT(major, minor)     ((__GNUC__ * 100 + __GNUC_MINOR__) < ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_LE(major, minor)     ((__GNUC__ * 100 + __GNUC_MINOR__) <= ((major) * 100 + (minor)))
#   define RF_COMPILER_STRING                       "gnu c/c++"
#   if  __GNUC__ == 2
#       if __GNUC_MINOR__ < 95
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ <2.95"
#       elif __GNUC_MINOR__ == 95
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 2.95"
#       elif __GNUC_MINOR__ == 96
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 2.96"
#       else
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ > 2.96 && < 3.0"
#       endif
#   elif __GNUC__ == 3
#       if __GNUC_MINOR__ == 2
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 3.2"
#       elif __GNUC_MINOR__ == 3
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 3.3"
#       elif __GNUC_MINOR__ == 4
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 3.4"
#       else
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ > 3.4 && < 4.0"
#       endif
#   elif __GNUC__ == 4
#       if __GNUC_MINOR__ == 1
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 4.1"
#       elif __GNUC_MINOR__ == 2
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 4.2"
#       elif __GNUC_MINOR__ == 3
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 4.3"
#       elif __GNUC_MINOR__ == 4
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 4.4"
#       elif __GNUC_MINOR__ == 5
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 4.5"
#       elif __GNUC_MINOR__ == 6
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 4.6"
#       elif __GNUC_MINOR__ == 7
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 4.7"
#       elif __GNUC_MINOR__ == 8
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 4.8"
#       elif __GNUC_MINOR__ == 9
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 4.9"
#       endif
#   elif __GNUC__ == 5
#       if __GNUC_MINOR__ == 1
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 5.1"
#       elif __GNUC_MINOR__ == 2
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 5.2"
#       elif __GNUC_MINOR__ == 3
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 5.3"
#       elif __GNUC_MINOR__ == 4
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 5.4"
#       elif __GNUC_MINOR__ == 5
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 5.5"
#       elif __GNUC_MINOR__ == 6
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 5.6"
#       elif __GNUC_MINOR__ == 7
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 5.7"
#       elif __GNUC_MINOR__ == 8
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 5.8"
#       elif __GNUC_MINOR__ == 9
#           define RF_COMPILER_VERSION_STRING       "gnu c/c++ 5.9"
#       endif
#   else
#       define RF_COMPILER_VERSION_STRING           "gnu Unknown"
#   endif

/* clang */
#   if defined(__clang__)
#       define RF_COMPILER_IS_CLANG
#       undef RF_COMPILER_STRING
#       define RF_COMPILER_STRING                   "clang c/c++"
#       if defined(__VERSION__)
#           undef RF_COMPILER_VERSION_STRING
#           define RF_COMPILER_VERSION_STRING       __VERSION__
#       elif defined(__clang_version__)
#           undef RF_COMPILER_VERSION_STRING
#           define RF_COMPILER_VERSION_STRING       __clang_version__
#       endif
/* ignore warning: empty struct has size 0 in C, size 1 in C++ */
#       ifdef __cplusplus
#           pragma clang diagnostic ignored         "-Wextern-c-compat"
#       endif
#   endif

/* watcom c/c++ */
#elif defined(__WATCOMC__)
#   define RF_COMPILER_IS_WATCOM
#   define RF_COMPILER_VERSION_BT(major, minor)     (__WATCOMC__ > ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_BE(major, minor)     (__WATCOMC__ >= ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_EQ(major, minor)     (__WATCOMC__ == ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_LT(major, minor)     (__WATCOMC__ < ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_LE(major, minor)     (__WATCOMC__ <= ((major) * 100 + (minor)))
#   define RF_COMPILER_STRING                       "watcom c/c++"
#   if (__WATCOMC__ == 1100)
#       define RF_COMPILER_VERSION_STRING           "watcom c/c++ 11.0"
#   elif (__WATCOMC__ == 1200)
#       define RF_COMPILER_VERSION_STRING           "open watcom c/c++ 1.0 (watcom 12.0)"
#   elif (__WATCOMC__ == 1210)
#       define RF_COMPILER_VERSION_STRING           "open watcom c/c++ 1.1 (watcom 12.1)"
#   elif (__WATCOMC__ == 1220)
#       define RF_COMPILER_VERSION_STRING           "open watcom c/c++ 1.2 (watcom 12.2)"
#   elif (__WATCOMC__ == 1230)
#       define RF_COMPILER_VERSION_STRING           "open watcom c/c++ 1.3 (watcom 12.3)"
#   elif (__WATCOMC__ == 1240)
#       define RF_COMPILER_VERSION_STRING           "open watcom c/c++ 1.4 (watcom 12.4)"
#   elif (__WATCOMC__ == 1250)
#       define RF_COMPILER_VERSION_STRING           "open watcom c/c++ 1.5"
#   elif (__WATCOMC__ == 1260)
#       define RF_COMPILER_VERSION_STRING           "open watcom c/c++ 1.6"
#   elif (__WATCOMC__ == 1270)
#       define RF_COMPILER_VERSION_STRING           "open watcom c/c++ 1.7"
#   else
#       define RF_COMPILER_VERSION_STRING           "open watcom Unknown"
#   endif

/* digital mars c/c++ */
#elif defined(__DMC__)
#   define RF_COMPILER_IS_DMC
#   define RF_COMPILER_VERSION_BT(major, minor)     (__DMC__ > ((major) * 256 + (minor)))
#   define RF_COMPILER_VERSION_BE(major, minor)     (__DMC__ >= ((major) * 256 + (minor)))
#   define RF_COMPILER_VERSION_EQ(major, minor)     (__DMC__ == ((major) * 256 + (minor)))
#   define RF_COMPILER_VERSION_LT(major, minor)     (__DMC__ < ((major) * 256 + (minor)))
#   define RF_COMPILER_VERSION_LE(major, minor)     (__DMC__ <= ((major) * 256 + (minor)))
#   define RF_COMPILER_STRING                       "digital mars c/c++"
#   if (__DMC__ < 0x0826)
#       error Only versions 8.26 and later of the digital mars c/c++ compilers are supported by the EXTL libraries
#   else
#       if __DMC__ >= 0x0832
#           define RF_COMPILER_VERSION_STRING       __DMC_VERSION_STRING__
#       elif (__DMC__ == 0x0826)
#           define RF_COMPILER_VERSION_STRING       "digital mars c/c++ 8.26"
#       elif (__DMC__ == 0x0827)
#           define RF_COMPILER_VERSION_STRING       "digital mars c/c++ 8.27"
#       elif (__DMC__ == 0x0828)
#           define RF_COMPILER_VERSION_STRING       "digital mars c/c++ 8.28"
#       elif (__DMC__ == 0x0829)
#           define RF_COMPILER_VERSION_STRING       "digital mars c/c++ 8.29"
#       elif (__DMC__ == 0x0830)
#           define RF_COMPILER_VERSION_STRING       "digital mars c/c++ 8.30"
#       elif (__DMC__ == 0x0831)
#           define RF_COMPILER_VERSION_STRING       "digital mars c/c++ 8.31"
#       else
#           define RF_COMPILER_VERSION_STRING       "digital Unknown"
#       endif
#   endif

/* codeplay vector c/c++ */
#elif defined(__VECTORC)
#   define RF_COMPILER_IS_VECTORC
#   define RF_COMPILER_VERSION_BT(major, minor)     (__VECTORC > (major))
#   define RF_COMPILER_VERSION_BE(major, minor)     (__VECTORC >= (major))
#   define RF_COMPILER_VERSION_EQ(major, minor)     (__VECTORC == (major))
#   define RF_COMPILER_VERSION_LT(major, minor)     (__VECTORC < (major))
#   define RF_COMPILER_VERSION_LE(major, minor)     (__VECTORC <= (major))
#   define RF_COMPILER_VERSION_STRING               "codeplay vector c/c++"
#   if (__VECTORC == 1)
#       define RF_COMPILER_VERSION_STRING           "codeplay vector c/c++"
#   else
#       define RF_COMPILER_VERSION_STRING           "codeplay Unknown"
#   endif

/* visual c++ */
#elif defined(_MSC_VER)
#   define RF_COMPILER_IS_MSVC
#   define RF_COMPILER_VERSION_BT(major, minor)     (_MSC_VER > ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_BE(major, minor)     (_MSC_VER >= ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_EQ(major, minor)     (_MSC_VER == ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_LT(major, minor)     (_MSC_VER < ((major) * 100 + (minor)))
#   define RF_COMPILER_VERSION_LE(major, minor)     (_MSC_VER <= ((major) * 100 + (minor)))
#       define RF_COMPILER_STRING                   "visual c++"
#   if defined(RF_FORCE_MSVC_4_2) && (_MSC_VER == 1020)
#       define RF_COMPILER_VERSION_STRING           "visual c++ 4.2"
#   elif (_MSC_VER == 1100)
#       define RF_COMPILER_VERSION_STRING           "visual c++ 5.0"
#   elif (_MSC_VER == 1200)
#       define RF_COMPILER_VERSION_STRING           "visual c++ 6.0"
#   elif (_MSC_VER == 1300)
#       define RF_COMPILER_VERSION_STRING           "visual c++ .net (7.0)"
#   elif (_MSC_VER == 1310)
#       define RF_COMPILER_VERSION_STRING           "visual c++ .net 2003 (7.1)"
#   elif (_MSC_VER == 1400)
#       define RF_COMPILER_VERSION_STRING           "visual c++ .net 2005 (8.0)"
#   elif (_MSC_VER == 1500)
#       define RF_COMPILER_VERSION_STRING           "visual c++ .net 2008 (9.0)"
#   elif (_MSC_VER == 1600)
#       define RF_COMPILER_VERSION_STRING           "visual c++ .net 2010 (10.0)"
#   elif (_MSC_VER == 1700)
#       define RF_COMPILER_VERSION_STRING           "visual c++ .net 2012 (11.0)"
#   elif (_MSC_VER == 1800)
#       define RF_COMPILER_VERSION_STRING           "visual c++ .net 2013 (12.0)"
#   elif (_MSC_VER == 1900)
#       define RF_COMPILER_VERSION_STRING           "visual c++ .net 2015 (14.0)"
#   else
#       define RF_COMPILER_VERSION_STRING           "visual c++ Unknown"
#   endif

#   pragma warning(disable:4018)
#   pragma warning(disable:4197)
#   pragma warning(disable:4141)
#   pragma warning(disable:4996)
#   pragma warning(disable:4359)
#   pragma warning(disable:4838)

#else
#   define RF_COMPILER_STRING                       "unknown compiler"
#   define RF_COMPILER_VERSION_STRING               "unknown compiler version"
#   define RF_COMPILER_IS_UNKNOWN
#endif

/* //////////////////////////////////////////////////////////////////////////////////////
 * macros
 */

/* arch
 *
 * gcc builtin macros for gcc -dM -E - < /dev/null
 *
 * .e.g gcc -m64 -dM -E - < /dev/null | grep 64
 * .e.g gcc -m32 -dM -E - < /dev/null | grep 86
 * .e.g gcc -march=armv6 -dM -E - < /dev/null | grep ARM
 */
#if defined(__i386) \
    || defined(__i686) \
    || defined(__i386__) \
    || defined(__i686__) \
    || defined(_M_IX86)
#   define RF_ARCH_x86
#   if defined(__i386) || defined(__i386__)
#       define  RF_ARCH_STRING              "i386"
#   elif defined(__i686) || defined(__i686__)
#       define  RF_ARCH_STRING              "i686"
#   elif defined(_M_IX86)
#       if (_M_IX86 == 300)
#           define  RF_ARCH_STRING          "i386"
#       elif (_M_IX86 == 400)
#           define  RF_ARCH_STRING          "i486"
#       elif (_M_IX86 == 500 || _M_IX86 == 600)
#           define  RF_ARCH_STRING          "Pentium"
#       endif
#   else
#       define RF_ARCH_STRING               "x86"
#   endif
#elif defined(__x86_64) \
    || defined(__amd64__) \
    || defined(__amd64) \
    || defined(_M_IA64) \
    || defined(_M_X64)
#   define RF_ARCH_x64
#   if defined(__x86_64)
#       define  RF_ARCH_STRING              "x86_64"
#   elif defined(__amd64__) || defined(__amd64)
#       define  RF_ARCH_STRING              "amd64"
#   else
#       define RF_ARCH_STRING               "x64"
#   endif
#elif defined(__arm__) || defined(__arm64) || defined(__arm64__) || (defined(__aarch64__) && __aarch64__)
#   define RF_ARCH_ARM
#   if defined(__ARM64_ARCH_8__)
#       define RF_ARCH_ARM64
#       define RF_ARCH_ARM_VERSION          (8)
#       define RF_ARCH_ARM_v8
#       define  RF_ARCH_STRING              "arm64"
#   elif defined(__ARM_ARCH_7A__)
#       define RF_ARCH_ARM_VERSION          (7)
#       define RF_ARCH_ARM_v7A
#       define  RF_ARCH_STRING              "armv7a"
#   elif defined(__ARM_ARCH_7__)
#       define RF_ARCH_ARM_VERSION          (7)
#       define RF_ARCH_ARM_v7
#       define  RF_ARCH_STRING              "armv7"
#   elif defined(__ARM_ARCH_6__)
#       define RF_ARCH_ARM_VERSION          (6)
#       define RF_ARCH_ARM_v6
#       define  RF_ARCH_STRING              "armv6"
#   elif defined(__ARM_ARCH_5TE__)
#       define RF_ARCH_ARM_VERSION          (5)
#       define RF_ARCH_ARM_v5te
#       define  RF_ARCH_STRING              "armv5te"
#   elif defined(__ARM_ARCH_5__)
#       define RF_ARCH_ARM_VERSION          (5)
#       define RF_ARCH_ARM_v5
#       define  RF_ARCH_STRING              "armv5"
#   elif defined(__ARM_ARCH)
#       define RF_ARCH_ARM_VERSION          __ARM_ARCH
#       if __ARM_ARCH >= 8
#           define RF_ARCH_ARM_v8
#           if defined(__arm64) || defined(__arm64__)
#               define RF_ARCH_ARM64
#               define RF_ARCH_STRING       "arm64"
#           else
#               define RF_ARCH_STRING       "armv7s"
#           endif
#       elif __ARM_ARCH >= 7
#           define RF_ARCH_ARM_v7
#           define  RF_ARCH_STRING          "armv7"
#       elif __ARM_ARCH >= 6
#           define RF_ARCH_ARM_v6
#           define  RF_ARCH_STRING          "armv6"
#       else
#           define RF_ARCH_ARM_v5
#           define RF_ARCH_STRING           "armv5"
#       endif
#   elif defined(__aarch64__) && __aarch64__
#       define RF_ARCH_ARM_v8
#       define RF_ARCH_ARM64
#       define RF_ARCH_STRING               "arm64-v8a"
#   else
#       define RF_ARCH_STRING               "arm"
#   endif
#   if !defined(RF_ARCH_ARM64) && (defined(__arm64) || defined(__arm64__))
#       define RF_ARCH_ARM64
#       ifndef RF_ARCH_STRING
#           define RF_ARCH_STRING           "arm64"
#       endif
#   endif
#   ifndef RF_ARCH_STRING
#       define RF_ARCH_STRING               "arm"
#   endif
#   if defined(__thumb__)
#       define RF_ARCH_ARM_THUMB
#       define RF_ARCH_STRING_2             "_thumb"
#   endif
#   if defined(__ARM_NEON__)
#       define RF_ARCH_ARM_NEON
#       define RF_ARCH_STRING_3             "_neon"
#   endif
#elif defined(mips) \
    || defined(_mips) \
    || defined(__mips__)
#   define RF_ARCH_MIPS
#   define RF_ARCH_STRING                   "mips"
#   elif defined(__C51__)
#   define RF_ARCH_STRING                   "c51"
#else
/*
#     define RF_ARCH_SPARC
#     define RF_ARCH_PPC
#     define RF_ARCH_SH4
*/
#   define RF_ARCH_STRING                   "unknown_arch"
#endif

/* sse */
#if defined(RF_ARCH_x86) || defined(RF_ARCH_x64)
#   if defined(__SSE__)
#       define RF_ARCH_SSE
#       define RF_ARCH_STRING_2             "_sse"
#   endif
#   if defined(__SSE2__)
#       define RF_ARCH_SSE2
#       undef RF_ARCH_STRING_2
#       define RF_ARCH_STRING_2             "_sse2"
#   endif
#   if defined(__SSE3__)
#       define RF_ARCH_SSE3
#       undef RF_ARCH_STRING_2
#       define RF_ARCH_STRING_2             "_sse3"
#   endif
#endif

/* vfp */
#if defined(__VFP_FP__)
#   define RF_ARCH_VFP
#   define RF_ARCH_STRING_4                 "_vfp"
#endif

/* elf */
#if defined(__ELF__)
#   define RF_ARCH_ELF
#   define RF_ARCH_STRING_5                 "_elf"
#endif

#if defined(__MACH__)
#   define RF_ARCH_MACH
#   define RF_ARCH_STRING_5                 "_mach"
#endif

#ifndef RF_ARCH_STRING_1
#   define RF_ARCH_STRING_1                 ""
#endif

#ifndef RF_ARCH_STRING_2
#   define RF_ARCH_STRING_2                 ""
#endif

#ifndef RF_ARCH_STRING_3
#   define RF_ARCH_STRING_3                 ""
#endif

#ifndef RF_ARCH_STRING_4
#   define RF_ARCH_STRING_4                 ""
#endif

#ifndef RF_ARCH_STRING_5
#   define RF_ARCH_STRING_5                 ""
#endif

#if defined(__LP64__) \
    || defined(__64BIT__) \
    || defined(_LP64) \
    || defined(__x86_64) \
    || defined(__x86_64__) \
    || defined(__amd64) \
    || defined(__amd64__) \
    || defined(__arm64) \
    || defined(__arm64__) \
    || defined(__sparc64__) \
    || defined(__PPC64__) \
    || defined(__ppc64__) \
    || defined(__powerpc64__) \
    || defined(_M_X64) \
    || defined(_M_AMD64) \
    || defined(_M_IA64) \
    || (defined(__WORDSIZE) && (__WORDSIZE == 64))
#   define RF_CPU_64BIT         (1)
#else
#   define RF_CPU_64BIT         (0)
#endif

#endif
