/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#ifndef __RF_TYPE_H__
#define __RF_TYPE_H__

#include <limits.h>

#ifndef TRUE
#define TRUE    1
#define FALSE   0
#endif

#ifndef NULL
#define NULL    0
#endif

#if RF_CPU_64BIT == 1
typedef unsigned long long      rf_size_t;
typedef signed long long        rf_ssize_t;
#else
typedef unsigned int            rf_size_t;
typedef int                     rf_ssize_t;
#endif
typedef char                    rf_tchar_t;
typedef char                    rf_char_t;
typedef short                   rf_wchar_t;
typedef short*                  rf_wchar_ref_t;
typedef int                     rf_int_t;
typedef void                    rf_void_t;
typedef void*                   rf_void_ref_t;
typedef void*                   rf_handle_t;
typedef unsigned char           rf_bool_t;
typedef unsigned char           rf_u8_t;
typedef unsigned char*          rf_u8_ref_t;
typedef unsigned short          rf_u16_t;
typedef unsigned short*         rf_u16_ref_t;
typedef unsigned int            rf_u32_t;
typedef unsigned int*           rf_u32_ref_t;
#if (ULLONG_MAX || __linux__)
#define RF_CL_HAVE_64BIT
typedef unsigned long long      rf_u64_t;
typedef unsigned long long*     rf_u64_ref_t;
typedef signed long long        rf_s64_t;
typedef signed long*            rf_s64_ref_t;
#endif

typedef signed char             rf_s8_t;
typedef signed char*            rf_s8_ref_t;
typedef signed short            rf_s16_t;
typedef signed short*           rf_s16_ref_t;
typedef signed long             rf_s32_t;
typedef signed long*            rf_s32_ref_t;

#if RF_OPT_GT_4G_SECTORS == 1
typedef unsigned long long      rf_lloff_t;
#else
typedef unsigned long           rf_lloff_t;
#endif

#if (RF_OPT_USE_EXFAT == 1 || RF_OPT_USE_NTFS == 1)
#ifndef RF_CL_HAVE_64BIT
#error "Compiler not support 64bit operation"
#endif
typedef  rf_s64_t               rf_off_t;
typedef  rf_u64_t               rf_fsize_t;
#else
typedef  rf_s32_t               rf_off_t;
typedef  rf_u32_t               rf_fsize_t;
#endif

#endif

