/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#ifndef __RF_NTFS_H__
#define __RF_NTFS_H__

/* ֽ֧ */
#define MAX_SECTOR_BYTE             4096
#define NTFS_SECTOR_BITS            9
#define NTFS_SECTOR_SIZE            512
#define IS_MAGIC(a,b)               (*(rf_u32_t*)(a) == *(rf_u32_t*)(b))

/* INDXĿ¼ һмֵ */
#define INDX_REPAIR_NUM             ( MAX_SECTOR_BYTE / NTFS_SECTOR_SIZE )

#define GET_CB_LEN( ret )           ( ret&0x0f )
#define GET_CB_OFFSET( ret )        ( ( ret >> 4 ) & 0x0f )
#define GET_BIT( d, n )             (((d) & (0x01<<(n)))==0?0:1)

/* NTFSֵ */
#define AT_FILE_HEAD                0x00
#define AT_STANDARD_INFORMATION     0x10
#define AT_ATTRIBUTE_LIST           0x20
#define AT_FILE_NAME                0x30
#define AT_VOLUME_VERSION           0x40
#define AT_SECURITY_DESCRIPTOR      0x50
#define AT_VOLUME_NAME              0x60
#define AT_VOLUME_INFORMATION       0x70
#define AT_DATA                     0x80
#define AT_INDEX_ROOT               0x90
#define AT_INDEX_ALLOCATION         0xA0
#define AT_BITMAP                   0xB0
#define AT_SYMLINK                  0xC0

rf_u64_t mft2sec(rf_ntfs_volume_ref_t vol, rf_u64_t mft);
rf_int_t rf_wstrncmp_buf(rf_u8_t* buf, rf_u16_t idx, rf_wchar_t* b, rf_u16_t n);
rf_s32_t get_s32(rf_u8_t* buf, rf_u16_t i, rf_u8_t cb);
rf_int_t ntfs_fixup_record(rf_ntfs_volume_ref_t vol, rf_char_t* magic);
rf_int_t ntfs_read_attr(rf_ntfs_volume_ref_t vol, rf_u64_t sector, rf_u8_t need_read,
                        rf_u8_t attr, rf_wchar_t* attr_name,  rf_u16_t* Idx);
rf_int_t ntfs_check_attrib_list(rf_ntfs_file_ref_t file, rf_u8_t attr, rf_wchar_t* attrib_name,
                                rf_u64_t* search_offset, rf_u64_t* mft_attrib_sector);
rf_int_t read_clust_chain(rf_ntfs_file_ref_t file, rf_u64_t sector, rf_u8_t need_read,
                          rf_u8_t attr, rf_wchar_t* attr_name, rf_u16_t* read_offset);
rf_int_t read_clust_chain_data(rf_ntfs_file_ref_t file, rf_void_t* buf, rf_u32_t* len, rf_int_t read);
rf_int_t check_index_bitmap(rf_ntfs_file_ref_t file, rf_bool_t* is_use_bit);
rf_int_t dir_read_offset_to_disk_buf(rf_ntfs_file_ref_t file, rf_u32_t offset_clust);
rf_int_t read_dir_to_disk_buf(rf_ntfs_file_ref_t file);
rf_int_t read_fixup_indx_sector(rf_ntfs_file_ref_t file);

#endif
