/*!The RanFs Library
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Copyright (C) 2014-2019, www.ranfs.com
 *
*/
#include "../include/rfs.h"
#define _RF_MAX_NAME 512
#include "../common/rf_common.h"
#include "ntfs_volume.h"
#include "ntfs_file.h"
#include "ntfs.h"

#define POSIX_NAME_SPACES           0
#define WIN32_NAME_SPACES           1
#define DOS_NAME_SPACES             2
#define WIN32_DOS_NAME_SPACES       3

#define DOS_NAME( n ) (n==DOS_NAME_SPACES || n==WIN32_DOS_NAME_SPACES)
#define WIN_DAME( n ) (n==WIN32_NAME_SPACES || n==WIN32_DOS_NAME_SPACES || n==POSIX_NAME_SPACES)

static rf_int_t _file_search(rf_ntfs_file_ref_t dir, const rf_wchar_t* name, rf_dirent_ref_t pfd);

static rf_int_t open_file(rf_ntfs_file_ref_t file)
{
    rf_int_t ret;
    rf_u16_t offset;
    rf_u16_t idx_offset;

    file->current_offset = 0;
    /* ĬбԴ0ʼ */
    file->list_search_offset = 0;
    /* б */
    file->list_run_bit = FALSE;

    idx_offset = 0;
    ret = ntfs_read_attr(file->vol, file->entry.sector, FALSE, AT_STANDARD_INFORMATION, NULL, &idx_offset);
    if (ret)
        return ret;
    offset = u16_read(file->vol->sector_buffer, (rf_u16_t)(idx_offset + 0x14));
    file->attrib = u32_read(file->vol->sector_buffer, (rf_u16_t)(idx_offset + offset + 0x20));

    idx_offset = 0;
    ret = ntfs_read_attr(file->vol, file->entry.sector, FALSE, AT_ATTRIBUTE_LIST, NULL, &idx_offset);
    if (!ret) {
        /* ҵб  */
        if (u8_read(file->vol->sector_buffer, idx_offset + 8) == 1)
            file->list_run_bit = TRUE;

        /* б õƫ ڵMFT */
        ret = ntfs_check_attrib_list(file, AT_DATA, NULL, &file->list_search_offset, &file->u.fi.mft_data_sector);
        /* ûĬ سɹ ֶ֧!! */
        if (ret == RF_ENOENT)
            ret = RF_ESUCCESS;
        if (ret)
            return ret;
    } else if (ret != RF_ENOENT)
        return ret;

    /* ļ ȡδ */
    idx_offset = 0;
    ret = ntfs_read_attr(file->vol, file->entry.sector, TRUE, AT_DATA, NULL, &idx_offset);
    if (ret) {
        /* ûҵδ */
        if (ret != RF_ENOENT)
            return ret;
        file->u.fi.length = 0;
        return RF_ESUCCESS;
    }
    /*  Ĭ϶MTFƫΪ0 */
    file->u.fi.data_body_offset = 0;
    /* Ĭϴ */
    file->data_run_bit = TRUE;
    if (u8_read(file->vol->sector_buffer, idx_offset + 8) == 0) {
        file->u.fi.length = u32_read(file->vol->sector_buffer, (rf_u16_t)(idx_offset + 0x10));
        file->data_run_bit = FALSE;
    } else {
        file->u.fi.length = u64_read(file->vol->sector_buffer, (rf_u16_t)(idx_offset + 0x30));
    }
    /* ʼ Ĭûռô */
    chain_init(&file->clust, TRUE);
    return RF_ESUCCESS;
}

rf_int_t open_dir(rf_ntfs_file_ref_t file)
{
    rf_int_t ret;
    rf_u16_t offset;
    rf_u16_t idx_offset;
    rf_wchar_t name[ 5 ];
    rf_u64_t list_offset;

    file->u.di.index_root_sector = file->entry.sector;
    file->u.di.index_alloc_sector = file->entry.sector;
    file->u.di.index_bitmap_sector = file->entry.sector;

    idx_offset = 0;
    ret = ntfs_read_attr(file->vol, file->entry.sector, FALSE, AT_STANDARD_INFORMATION, NULL, &idx_offset);
    if (ret)
        return ret;
    offset = u16_read(file->vol->sector_buffer, (rf_u16_t)(idx_offset + 0x14));
    file->attrib = u32_read(file->vol->sector_buffer, (rf_u16_t)(idx_offset + offset + 0x20));
    /* MFTԱ־λ ܴڲΪļ Աļ */
    file->attrib |= RF_ATTR_DIRECTORY;

    idx_offset = 0;
    ret = ntfs_read_attr(file->vol, file->entry.sector, FALSE, AT_ATTRIBUTE_LIST, NULL, &idx_offset);
    if (ret == RF_ESUCCESS) {
        /* ҵб  */
        if (u8_read(file->vol->sector_buffer, idx_offset + 8) == 1)
            file->list_run_bit = TRUE;

        list_offset = 0;
        /* Ŀ¼ б õ ڵMFT */
        rf_a2w_strcpy(name, "$I30");
        ret = ntfs_check_attrib_list(file, AT_INDEX_ROOT, name, &list_offset, &file->u.di.index_root_sector);
        if (ret)
            return ret;

        /* ² õMFT */
        ret = ntfs_check_attrib_list(file, AT_INDEX_ALLOCATION, name, &list_offset, &file->u.di.index_alloc_sector);
        if (ret && ret != RF_ENOENT)
            return ret;
        file->list_search_offset = list_offset;

        /* ² õλͼMFT */
        ret = ntfs_check_attrib_list(file, AT_BITMAP, name, &list_offset, &file->u.di.index_bitmap_sector);
        if (ret && ret != RF_ENOENT)
            return ret;
    } else if (ret != RF_ENOENT)
        return ret;

    /* ȡ */
    rf_a2w_strcpy(name, "$I30");
    idx_offset = 0;
    ret = ntfs_read_attr(file->vol, file->u.di.index_root_sector, TRUE, AT_INDEX_ROOT, name, &idx_offset);
    if (ret)
        return ret;

    /* µõԵĿʼַ */
    offset = u16_read(file->vol->sector_buffer, (rf_u16_t)(idx_offset + 0x14));
    offset += idx_offset;

    /* õһ¼ռü */
    file->u.di.clust_per_index_record_bits = rf_ffs(u8_read(file->vol->sector_buffer, offset + 0x0c));

    /* Index Root 8ֽڶ µõ Index Header ʼַ */
    offset +=  0x10 + (offset % 8);

    /* 0x00 Small Index (fits in Index Root)  */
    file->data_run_bit = FALSE;
    if (u8_read(file->vol->sector_buffer, offset + 0x0c))
        file->data_run_bit = TRUE;

    file->u.di.enum_vcn_index = 0;
    file->u.di.index_alloc_body_offset = 0;
    file->u.di.enum_last_offset = 0;
    file->u.di.enum_idx_root_flags = FALSE;
    /* ʼ Ĭûռô */
    chain_init(&file->clust, TRUE);
    /* Ĭ϶ȡ ļ */
    file->ctx = (rf_void_ref_t)RF_NAME_WIN32;
    return RF_ESUCCESS;
}

rf_int_t ntfs_file_raw_open(rf_ntfs_file_ref_t file, rf_fentry_t* fentry, rf_u32_t open_flag)
{
    rf_int_t ret ;
    rf_bool_t is_dir_bit;
    rf_u16_t idx_offset;

    /* ļͷ */
    idx_offset = 0;
    ret = ntfs_read_attr(file->vol, fentry->sector, TRUE, AT_FILE_HEAD, NULL, &idx_offset);
    if (ret)
        return ret;
    /* ļѾɾ ش򿪴 */
    if (!(u8_read(file->vol->sector_buffer, 0x16) & 0x01))
        return RF_ERROR;
    /* жǲĿ¼ */
    is_dir_bit = FALSE;
    if (u8_read(file->vol->sector_buffer, 0x16) & 0x02)
        is_dir_bit = TRUE;

    file->entry.sector = fentry->sector;
    if (!is_dir_bit) {
        return open_file(file);
    } else {
        return open_dir(file);
    }
}

static rf_int_t _file_open(rf_ntfs_file_ref_t file, const rf_wchar_t* name, rf_u32_t open_flag)
{
    rf_int_t ret;
    rf_dirent_t fdir;

    fdir.name = NULL;
    fdir.len = 0;
    ret = _file_search(file, name, &fdir);
    if (ret != RF_ESUCCESS)
        return ret;
    file->entry.sector = fdir.entry.sector;
    file->entry.offset = fdir.entry.offset;
    return ntfs_file_raw_open(file, &file->entry, open_flag);
}

rf_int_t ntfs_file_open(rf_ntfs_file_ref_t file, const rf_tchar_t* name, rf_int_t flags)
{
    rf_int_t ret;
    rf_int_t last_path = 0;
    const rf_tchar_t* fn;
    rf_fentry_t fentry;
    rf_wchar_t* buf;
    DEF_PATH_BUF

    fn = name;
    file->access_flag = flags & 0x3;
    /*  ֧· 򿪷ǰĿ¼ */
    if (is_absolute_path(name)) {
        fentry = file->vol->root_dir_entry;
    } else {
        fentry = file->vol->current_dir_entry;
    }
    ret = ntfs_file_raw_open(file, &fentry, RF_O_RDONLY);
    if (ret != RF_ESUCCESS)
        return ret;

    GET_PATH_BUF(buf);
    do {
        ret = path_get_fn(&fn, (rf_char_t*)buf, RF_MAX_FNAME, RF_CODE_PAGE_UCS2L);
        if (ret != RF_ESUCCESS)
            break;
        ret = path_is_last(&fn, &last_path);
        if (ret != RF_ESUCCESS)
            break;
        /* ȡļ */
        if (buf[0] == '\0')
            break;
        /* 򿪵ǵǰĿ¼  */
        if (buf[0] ==  '.' && buf[1] == '\0')
            continue;

        if (last_path == 1) {
            ret = _file_open(file, (rf_wchar_t*)buf, flags);
        } else { /* мΪĿ¼ ʹֻ  */
            ret = _file_open(file, (rf_wchar_t*)buf, RF_O_RDONLY);
        }

        if (ret != RF_ESUCCESS) {
            /* һĿ¼ ļĿ¼ */
            if (ret == RF_ENDFILE && last_path == 1 && flags & RF_O_CREAT)
                ret = RF_EPERM;
            break;
        }

        /* ֻһ ļ  */
        if (last_path == 0 && !(file->attrib & RF_ATTR_DIRECTORY)) {
            RF_PRINTF(RF_DBG_LEVEL_ERROR, ("Path to the include file error\n"));
            ret = RF_ENOTDIR;
            break;
        }

    } while (last_path == 0);
    FREE_PATH_BUF(buf);
    if (ret)
        return ret;

    if (file->attrib & RF_ATTR_DIRECTORY)
        return RF_ESUCCESS;
    return  RF_ESUCCESS;
}

rf_int_t ntfs_file_lseek(rf_ntfs_file_ref_t file, rf_u64_t offset, rf_int_t origin)
{
    return -1;
}

rf_int_t ntfs_file_read(rf_ntfs_file_ref_t file, rf_void_t* buf, rf_u32_t len, rf_u32_t* plen)
{
    rf_int_t ret;
    rf_u8_t* tbuf;
    rf_u16_t idx_offset, offset;
    rf_u32_t tmp_len, count_len;

    if (file->access_flag == RF_O_WRONLY)
        return RF_EACCES;
    if (file->sector_buf_flag) {
        if (len != file->vol->sector_size)
            RF_PERROR("prameter error\n", 1, return RF_EINVAL;);
    }

    /* ԼļѹļĿ¼ жȡ ش */
    /* Ŀ¼ȡӦĿ¼ȡ */
    if (file->attrib & RF_ATTR_ENCRYPTED || file->attrib & RF_ATTR_COMPRESSED || file->attrib & RF_ATTR_DIRECTORY) {
        *plen = 0;
        RF_PERROR("Does not support the file read operation\n", 1, return RF_ESUCCESS;);
    }

    /* Ŀ¼ļ ûļȴС м ؽΪĿ¼ */
    if (!(file->attrib & RF_ATTR_DIRECTORY)) {
        if ((file->current_offset + len) > file->u.fi.length)
            len = (rf_u32_t)(file->u.fi.length - file->current_offset);
    }
    if (len == 0) {
        *plen = 0;
        return RF_ESUCCESS;
    }

    if (!file->data_run_bit) {
        if (buf) {
            idx_offset = 0;
            ret = ntfs_read_attr(file->vol, file->entry.sector, TRUE, AT_DATA, NULL, &idx_offset);
            if (ret)
                return ret;
            /* Դ ֱӽݸ */
            offset = u16_read(file->vol->sector_buffer, (rf_u16_t)(idx_offset + 0x14));
            offset += idx_offset + (rf_u16_t)(file->current_offset);
            /* Ӧüȡݵĳ!!! */
            memcpy(buf, file->vol->sector_buffer + offset, len);
            *plen = len;
        }
        file->current_offset += len;
    } else {
        /*  ʼȡļ ^_^ */
        count_len = len;
        tbuf = buf;
        *plen = 0;
        while (count_len) {
            /* ڴ˼ǷѾռ ռĴѾ */
            if (file->clust.last_chain_index == 0 || file->clust.chain_offset >= file->clust.last_chain_index) {
                if (file->u.fi.data_body_offset != USHRT_MAX) {
                    /* ȡص */
                    ret = read_clust_chain(file, file->entry.sector, TRUE, AT_DATA, NULL, &file->u.fi.data_body_offset);
                    if (ret == RF_ENDFILE)
                        continue;
                    if (ret)
                        return ret;
                } else {
                    /* ######### */
                    ret = ntfs_check_attrib_list(file, AT_DATA, NULL, &file->list_search_offset, &file->u.fi.mft_data_sector);
                    if (ret)
                        return ret;

                    file->u.fi.data_body_offset = 0;
                    /* ٴλȡõ */
                    ret = read_clust_chain(file, file->entry.sector, TRUE, AT_DATA, NULL, &file->u.fi.data_body_offset);
                    if (ret)
                        return ret;
                }
            }

            /* Ѵݶ */
            tmp_len = count_len;
            ret = read_clust_chain_data(file, tbuf, &tmp_len, (rf_int_t)(buf != NULL ? TRUE : FALSE));
            /*  F_ERR ΪƫƵβ ¶´ص */
            if (ret && ret != RF_ENDFILE)
                return ret;

            /* ıʵʶȡݳ */
            *plen += tmp_len;
            /* ı仺ƫ */
            tbuf += tmp_len;
            /* дǰļƫ */
            file->current_offset += tmp_len;
            /* ʣݳ Ҫȡĳ */
            count_len -= tmp_len;
            tmp_len = count_len;
        }
    }
    return RF_ESUCCESS;
}

rf_int_t ntfs_file_close(rf_ntfs_file_ref_t file)
{
    file->open_flag = FALSE;
    return RF_ESUCCESS;
}

rf_int_t ntfs_file_get_stat(rf_ntfs_file_ref_t file, rf_stat_ref_t st)
{
    st->entry.sector = file->entry.sector;
    st->entry.offset = file->entry.offset;
    st->st_mode = file->attrib;
    st->st_size = file->u.fi.length;
    st->current_offset = file->current_offset;
    return RF_ESUCCESS;
}

rf_int_t ntfs_file_get_ftime(rf_ntfs_file_ref_t file, rf_file_ftime_ref_t ft)
{
    rf_int_t ret;
    rf_u16_t offset;
    rf_u16_t idx_offset;
    rf_u64_t creation_time, modified_time, access_time;

    idx_offset = 0;
    ret = ntfs_read_attr(file->vol, file->entry.sector, FALSE, AT_STANDARD_INFORMATION, NULL, &idx_offset);
    if (ret)
        return ret;

    offset = u16_read(file->vol->sector_buffer, (rf_u16_t)(idx_offset + 0x14));
    creation_time = u64_read(file->vol->sector_buffer, (rf_u16_t)(idx_offset + offset + 0x00));
    modified_time = u64_read(file->vol->sector_buffer, (rf_u16_t)(idx_offset + offset + 0x10));
    access_time = u64_read(file->vol->sector_buffer, (rf_u16_t)(idx_offset + offset + 0x18));
    ft->create.low_date_time = (rf_u32_t)creation_time;
    ft->create.high_date_time = (rf_u32_t)(creation_time >> 32);
    ft->last_write.low_date_time = (rf_u32_t)modified_time;
    ft->last_write.high_date_time = (rf_u32_t)(modified_time >> 32);
    ft->last_access.low_date_time = (rf_u32_t)access_time;
    ft->last_access.high_date_time = (rf_u32_t)(access_time >> 32);
    return 0;
}

static rf_int_t ntfs_get_file_attrib(rf_ntfs_volume_ref_t vol, rf_u64_t mft, rf_u32_t* attrib)
{
    rf_int_t ret;
    rf_u16_t offset, idx_offset;
    rf_bool_t is_dir_bit;

    idx_offset = 0;
    ret = ntfs_read_attr(vol, mft, TRUE, AT_FILE_HEAD, NULL, &idx_offset);
    if (ret)
        return ret;
    is_dir_bit = FALSE;
    if (u8_read(vol->sector_buffer, 0x16) & 0x02)
        is_dir_bit = TRUE;

    ret = ntfs_read_attr(vol, mft, FALSE, AT_STANDARD_INFORMATION, NULL, &idx_offset);
    if (ret)
        return ret;
    offset = u16_read(vol->sector_buffer, (rf_u16_t)(idx_offset + 0x14));
    *attrib = u32_read(vol->sector_buffer, (rf_u16_t)(idx_offset + offset + 0x20));
    if (is_dir_bit)
        *attrib |= RF_ATTR_DIRECTORY;
    return RF_ESUCCESS;
}

rf_int_t _enum_next_file(rf_ntfs_file_ref_t file, const rf_wchar_t* name, rf_dirent_ref_t pfd, rf_wchar_t* buf)
{
    rf_int_t i;
    rf_u8_t len, name_space;
    rf_u64_t mft;

    for (;;) {
        mft = u64_read(file->vol->sector_buffer, (rf_u16_t)(file->u.di.enum_last_offset + 0));
        if (mft == 0)
            return RF_ENDFILE;

        name_space = u8_read(file->vol->sector_buffer, (rf_u16_t)(file->u.di.enum_last_offset  + 0x51));
        if (((rf_u8_t)file->ctx == RF_NAME_WIN32 && !WIN_DAME(name_space))
                || ((rf_u8_t)file->ctx == RF_NAME_DOS && !DOS_NAME(name_space))) {
            file->u.di.enum_last_offset += u16_read(file->vol->sector_buffer, (rf_u16_t)(file->u.di.enum_last_offset + 0x08));
            continue;
        }

        len = u8_read(file->vol->sector_buffer, (rf_u16_t)(file->u.di.enum_last_offset  + 0x50));
        for (i = 0; i < len; i++)
            buf[i] = u16_read(file->vol->sector_buffer, (rf_u16_t)(file->u.di.enum_last_offset + 0x52 + (i << 1)));
        buf[len] = 0;

        file->u.di.enum_last_offset += u16_read(file->vol->sector_buffer, (rf_u16_t)(file->u.di.enum_last_offset + 0x08));
        if (rf_wstr_in_pattern(buf, name) && pfd) {
            pfd->entry.sector = mft2sec(file->vol, mft);
            pfd->entry.offset = 0;
            i = ntfs_get_file_attrib(file->vol, pfd->entry.sector, &pfd->attribute);
            if (i)
                return i;
            if (pfd->name)
                rf_wstrcpy((rf_wchar_t*)pfd->name, buf);
            pfd->type = RF_CODE_PAGE_UCS2L;
            return RF_ESUCCESS;
        }
    }
}

static rf_int_t enum_idx_next_file(rf_ntfs_file_ref_t file, const rf_wchar_t* name, rf_dirent_ref_t pfd, rf_wchar_t* buf)
{
    rf_int_t ret;
    rf_u64_t mft;
    rf_u32_t i, tmp, offset, len, attr_len;
    rf_u8_t name_space;
    rf_bool_t next_sector_bit;

    for (;;) {
        tmp = file->u.di.enum_last_offset % file->vol->sector_size;
        mft = u64_read(file->vol->sector_buffer, (rf_u16_t)(tmp + 0));
        if (mft == 0)
            return RF_ENDFILE;

        offset = tmp + 0x08;
        next_sector_bit = FALSE;
        if (offset >= file->vol->sector_size) {
            next_sector_bit = TRUE;
            ret = read_dir_to_disk_buf(file);
            if (ret)
                return ret;
            offset %= file->vol->sector_size;
        }
        attr_len = u16_read(file->vol->sector_buffer, (rf_u16_t)offset);

        offset = tmp + 0x50;
        if (offset >= file->vol->sector_size) {
            if (!next_sector_bit) {
                ret = read_dir_to_disk_buf(file);
                if (ret)
                    return ret;
                next_sector_bit = TRUE;
            }
            offset %= file->vol->sector_size;
        }
        len = u8_read(file->vol->sector_buffer, (rf_u16_t)offset);
        name_space = u8_read(file->vol->sector_buffer, (rf_u16_t)(offset + 1));

        if (((rf_u8_t)file->ctx == RF_NAME_WIN32 && !WIN_DAME(name_space))
                || ((rf_u8_t)file->ctx == RF_NAME_DOS && !DOS_NAME(name_space))) {
            if ((offset + attr_len - 0x50) >= file->vol->sector_size) {
                /* һ */
                ret = read_dir_to_disk_buf(file);
                if (ret)
                    return ret;
            }
            file->u.di.enum_last_offset += attr_len;
            continue;
        }

        /* ʼȡļ */
        tmp = offset + 0x2 + (len << 1);
        if (tmp <= file->vol->sector_size) {
            /* ڵһ */
            for (i = 0; i < len; i++)
                buf[i] = u16_read(file->vol->sector_buffer, (rf_u16_t)(offset + 0x2 + (i << 1)));
            buf[len] = 0;
            if ((offset + attr_len - 0x50) >= file->vol->sector_size) {
                /* һ */
                ret = read_dir_to_disk_buf(file);
                if (ret)
                    return ret;
            }
        } else {
            /* ļ */
            len = (file->vol->sector_size - (offset + 0x2)) >> 1;
            for (i = 0; i < len; i++)
                buf[i] = u16_read(file->vol->sector_buffer, (rf_u16_t)(offset + 0x2 + (i << 1)));
            /* һ */
            ret = read_dir_to_disk_buf(file);
            if (ret)
                return ret;
            tmp %= file->vol->sector_size;
            tmp >>= 1;
            for (i = 0; i < tmp; i++)
                buf[ len + i] = u16_read(file->vol->sector_buffer, (rf_u16_t)(0 + (i << 1)));
            buf[len + tmp] = 0;
        }

        file->u.di.enum_last_offset += attr_len;
        if (rf_wstr_in_pattern(buf, name) && pfd) {
            pfd->entry.sector = mft2sec(file->vol, mft);
            pfd->entry.offset = 0;
            ret = ntfs_get_file_attrib(file->vol, pfd->entry.sector, &pfd->attribute);
            if (ret)
                return ret;
            /* Ҫfat ansi ļ */
            if (pfd->name != NULL)
                rf_wstrcpy((rf_wchar_t*)pfd->name, buf);
            pfd->type = RF_CODE_PAGE_UCS2L;
            return RF_ESUCCESS;
        }
    }
}

static rf_int_t enum_idx_root_file(rf_ntfs_file_ref_t file, const rf_wchar_t* name, rf_dirent_ref_t pfd, rf_wchar_t* buf)
{
    rf_int_t ret;
    rf_u32_t max_offset;
    rf_u16_t idx_offset, offset;

    rf_a2w_strcpy(buf, "$I30");
    idx_offset = 0;
    ret = ntfs_read_attr(file->vol, file->u.di.index_root_sector, TRUE, AT_INDEX_ROOT, buf, &idx_offset);
    if (ret)
        return ret;

    if (u8_read(file->vol->sector_buffer, idx_offset + 8) == 0) {
        /*log_error( "AT_INDEX_ROOT at run data\n" );
        return RF_ERROR;*/
    }

    if (file->u.di.enum_last_offset == 0) {
        offset = u16_read(file->vol->sector_buffer, (rf_u16_t)(idx_offset + 0x14));
        offset += idx_offset;
        /* Index Root 8ֽڶ µõ Index Header ʼַ */
        offset +=  0x10 + (offset % 8);
        file->u.di.enum_last_offset = offset + u32_read(file->vol->sector_buffer, (rf_u16_t)(offset + 0));
        max_offset = offset + u32_read(file->vol->sector_buffer, (rf_u16_t)(offset + 4));
    }
    return _enum_next_file(file, name, pfd, buf);
}

static rf_int_t _file_search(rf_ntfs_file_ref_t file, const rf_wchar_t* name, rf_dirent_ref_t pfd)
{
    rf_int_t ret = 0;
    rf_u32_t idx_offset, offset, record_sector;
    rf_bool_t use_bit;
    rf_wchar_t* buf;
    DEF_FIND_BUF

    /* Ŀ¼֧ö 򷵻ش */
    if (!(file->attrib & RF_ATTR_DIRECTORY))
        return RF_ENOTDIR;

    GET_FIND_BUF(buf);
    if (!file->data_run_bit) {
        rf_a2w_strcpy(buf, "$I30");
        idx_offset = 0;
        ret = ntfs_read_attr(file->vol, file->entry.sector, TRUE, AT_INDEX_ROOT, buf, (rf_u16_t*)&idx_offset);
        if (ret)
            goto _EXIT;

        offset = u16_read(file->vol->sector_buffer, (rf_u16_t)(idx_offset + 0x14));
        offset += idx_offset;
        /* Index Root 8ֽڶ µõ Index Header ʼַ */
        offset += 0x10 + (offset % 8);
        idx_offset = offset + u32_read(file->vol->sector_buffer, (rf_u16_t)(offset + 0));
        file->u.di.enum_clust_max_offset = offset + u32_read(file->vol->sector_buffer, (rf_u16_t)(offset + 4));
        if (file->u.di.enum_last_offset == 0)
            file->u.di.enum_last_offset = idx_offset;
        ret = _enum_next_file(file, name, pfd, buf);
        goto _EXIT;
    } else {
        /*  B+ ٶλָ!!! */
        if (!file->u.di.enum_idx_root_flags) {
            ret = enum_idx_root_file(file, name, pfd, buf);
            if (ret == RF_ESUCCESS)
                goto _EXIT;
            if (ret != RF_ENDFILE)
                goto _EXIT;
            file->u.di.enum_last_offset = 0;
            file->u.di.enum_idx_root_flags = TRUE;
        }

        for (;;) {
            if (!file->u.di.enum_last_offset) {
                /* һν ʼöļ */
                offset = 0;
                for (;;) {
                    use_bit = FALSE;
                    ret = check_index_bitmap(file, &use_bit);
                    if (ret)
                        goto _EXIT;
                    if (use_bit)
                        break;
                    file->u.di.enum_vcn_index++;
                    offset++;
                }
                /* ¼ öٴƫ */
                file->u.di.enum_record_sector_offset = 0;
                /* Ӧƫƴ */
                offset <<= file->u.di.clust_per_index_record_bits;
                /* ǰƫƴ Ȼȡ */
                ret = dir_read_offset_to_disk_buf(file, offset);
                if (ret)
                    break;

                if (IS_MAGIC(file->vol->sector_buffer, "\0\0\0\0")) {
                    ret = RF_ENDFILE;
                    break;
                }
                /* ʼöļ ÿؿʼһ Ҫ⴦ */
                if (!IS_MAGIC(file->vol->sector_buffer, "INDX")) {
                    ret = RF_ERROR;
                    break;
                }
                file->u.di.enum_last_offset = 0x18 + u32_read(file->vol->sector_buffer, 0x18);
                file->u.di.enum_clust_max_offset = 0x18 + u32_read(file->vol->sector_buffer, 0x1c);
            } else {
                /* ڶνöļ */
                /* ƫһ Ȼȡһζȡ */
                chain_back_offset_sector(&file->clust, file->vol->sectors_per_cluster, 1);
                /* ȡݵ  */
                ret = read_fixup_indx_sector(file);
                if (ret)
                    break;
            }

            ret = enum_idx_next_file(file, name, pfd, buf);
            /* ǰ öļ  */
            if (ret == RF_ENDFILE) {
                /* õѾ¼ʣ */
                record_sector = file->vol->sectors_per_cluster << file->u.di.clust_per_index_record_bits;
                record_sector -= (file->u.di.enum_record_sector_offset + 1);
                /* ʣ ƫƵһ¼ʼ */
                if (record_sector)
                    chain_offset_sector(&file->clust, file->vol->sectors_per_cluster, record_sector);

                /* شӿʼöļ */
                file->u.di.enum_last_offset = 0;
                /* ʼ ͬʱdir.enum_vcn_index++*/
                file->u.di.enum_vcn_index++;
                continue;
            }
            break;
        }
    }
_EXIT:
    FREE_FIND_BUF(buf);
    return ret;
}

rf_int_t ntfs_file_read_dir(rf_ntfs_file_ref_t file, const rf_char_t* pattern, rf_dirent_ref_t pfd)
{
    rf_wchar_t name[2];

    name[0] = '*';
    name[1] = '\0';
    return _file_search(file, name, pfd);
}

#if RF_OPT_USE_GET_FNAME
rf_int_t ntfs_file_get_name(rf_ntfs_file_ref_t file, rf_char_t* buf, rf_u32_t len)
{
    return 0;
}
#endif

#if RF_OPT_USE_GET_SET_LONG_NAME
rf_int_t ntfs_file_get_long_name(rf_ntfs_file_ref_t file, rf_wchar_t* buf, rf_u32_t len)
{
    return 0;
}
#endif
