#include <windows.h>
#include <winioctl.h>
#include <stdio.h>
#include "rfs.h"
#include "win32_drv.h"
#include "liblog.h"

static rf_int_t get_sector_size(rf_handle_t handle, rf_u32_t* size);
rf_int_t disk_drv_open(rf_handle_t dev, const rf_char_t* dev_name, rf_int_t flags)
{
    TCHAR win_drv_name[ MAX_PATH ];
    DWORD dwFlagsAndAttributes;
    rf_handle_t handle;
    rf_disk_context_ref_t ctx = (rf_disk_context_ref_t)dev;

    sprintf_s(win_drv_name, MAX_PATH, "%s", dev_name);
    dwFlagsAndAttributes = FILE_ATTRIBUTE_NORMAL;
    if (flags & RF_O_NOCACHE)
        dwFlagsAndAttributes |= FILE_FLAG_NO_BUFFERING;

    if ((flags & 0x3) == 2) {
        /* дļ  ֻһд */
        handle = (rf_handle_t)CreateFile(win_drv_name, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ,
                                         NULL, OPEN_EXISTING, dwFlagsAndAttributes, NULL);
    } else {
        /*  ̶д */
        handle = (rf_handle_t)CreateFile(win_drv_name, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE,
                                         NULL, OPEN_EXISTING, dwFlagsAndAttributes, NULL);
    }

    if (handle == INVALID_HANDLE_VALUE) {
        log_perror("CreateFile:%s error:", win_drv_name);
        return RF_EIO;
    }
    ctx->handle = handle;
    return get_sector_size(handle, &ctx->sector_size);
}

rf_int_t disk_drv_close(rf_handle_t dev)
{
    rf_disk_context_ref_t ctx = (rf_disk_context_ref_t)dev;

    return CloseHandle(ctx->handle) ? 0 : 1;
}

static rf_int_t get_sector_size(rf_handle_t handle, rf_u32_t* size)
{
    DWORD dwByteReturned;
    DISK_GEOMETRY Geometry;

    dwByteReturned = 0;
    ZeroMemory(&Geometry, sizeof(Geometry));
    if (!DeviceIoControl(
                handle,
                IOCTL_DISK_GET_DRIVE_GEOMETRY,
                NULL,
                0,
                &Geometry,
                sizeof(Geometry),
                &dwByteReturned,
                NULL
            )) {
        log_perror("DeviceIoControl:IOCTL_DISK_GET_DRIVE_GEOMETRY error:");
        return RF_EIO;
    }
    *size = Geometry.BytesPerSector;
    return 0;
}

rf_int_t disk_drv_ioctl(rf_handle_t dev, rf_u32_t cmd, rf_void_t* buf, rf_u32_t len)
{
    rf_disk_context_ref_t ctx = (rf_disk_context_ref_t)dev;

    switch (cmd) {
    case RF_CMD_GET_SECTOR_SIZE:
        if (len != sizeof(rf_u32_t))
            return RF_EINVAL;
        *((rf_u32_t*)buf) = ctx->sector_size;
        return TRUE;

    case RF_CMD_GET_SECTOR_COUNT:
        return RF_EINVAL;

    default:
        return RF_EINVAL;
    }
}

rf_int_t disk_drv_read(rf_handle_t dev, rf_lloff_t sector_offset, rf_u32_t sector_count, rf_void_t* buf)
{
    rf_disk_context_ref_t ctx = (rf_disk_context_ref_t)dev;
    rf_u32_t len, dwByteReturned = 0;
    LARGE_INTEGER li;

    li.QuadPart = (ctx->volume_offset + (rf_u64_t)sector_offset) * ctx->sector_size;
    if (!SetFilePointerEx(ctx->handle, li, NULL, FILE_BEGIN)) {
        log_perror("SetFilePointerEx: ");
        return RF_EIO;
    }
    if (!ReadFile(ctx->handle, buf, (DWORD)(sector_count * ctx->sector_size), (LPDWORD)&len, NULL)) {
        log_perror("ReadFile: ");
        return RF_EIO;
    }
    return 0;
}

rf_int_t disk_drv_write(rf_handle_t dev, rf_lloff_t sector_offset, rf_u32_t sector_count, rf_void_t* buf)
{
    rf_disk_context_ref_t ctx = (rf_disk_context_ref_t)dev;
    rf_u32_t len;
    LARGE_INTEGER li;

    li.QuadPart = (ctx->volume_offset + (rf_u64_t)sector_offset) * ctx->sector_size;
    if (!SetFilePointerEx(ctx->handle, li, NULL, FILE_BEGIN)) {
        log_perror("SetFilePointerEx: ");
        return RF_EIO;
    }
    if (!WriteFile(ctx->handle, buf, (DWORD)(sector_count * ctx->sector_size), (LPDWORD)&len, NULL)) {
        log_perror("WriteFile: ");
        return RF_EIO;
    }
    return 0;
}
