#include <stdio.h>
#include <windows.h>
#include <winioctl.h>
#include "rfs.h"
#include "win32_drv.h"
#include "file_drv.h"
#include "disk_drv.h"
#ifdef RF_OPT_TOOL_MODE
#include "name.h"
#endif

#define TYPE_FILE       1
#define TYPE_DISK       2
#define TYPE_VMDK       3

rf_int_t win_open(const rf_char_t* device_name, rf_int_t flags, rf_void_t* ctx, rf_handle_t* hdev)
{
    rf_int_t ret;
    rf_disk_context_ref_t dev_ctx;
    rf_char_t* sep, *_name;
    rf_mount_ctx_ref_t m_ctx = (rf_mount_ctx_ref_t)ctx;
    rf_char_t buf[RF_MAX_PATH];
    rf_char_t real_dev_name[RF_MAX_PATH];

    dev_ctx = (rf_disk_context_ref_t)malloc(sizeof(rf_disk_context_t));
    if (dev_ctx == NULL)
        return RF_ERROR;
    memset(dev_ctx, 0, sizeof(*dev_ctx));

    strcpy_s(buf, sizeof(buf), device_name);
    sep = strchr(buf, '\t');
    if (sep == NULL)
        return RF_EINVAL;
    *sep = 0;

    // ȡļʵļ
    _name = sep + 1;
#ifdef RF_OPT_TOOL_MODE
    convert_url_real_name(_name, real_dev_name, sizeof(real_dev_name));

    if (!strcmp(buf, "check"))
        strcpy(buf, get_url_type(_name));
#else
    strcpy_s(real_dev_name, sizeof(real_dev_name), _name);
#endif
    if (!strcmp(buf, "file"))
        dev_ctx->type = TYPE_FILE;
    else if (!strcmp(buf, "disk") || !strcmp(buf, "voloume"))
        dev_ctx->type = TYPE_DISK;
    else if (!strcmp(buf, "vmdk"))
        dev_ctx->type = TYPE_VMDK;

    // ʵ豸ж!!
    switch (dev_ctx->type) {
    case TYPE_FILE:
        ret = file_drv_open(dev_ctx, real_dev_name, flags);
        break;

    case TYPE_DISK:
        ret = disk_drv_open(dev_ctx, real_dev_name, flags);
        break;

    case TYPE_VMDK:
        //ret = vmdk_drv_open(dev_ctx, real_dev_name, flags);
        break;

    default:
        ret = RF_EINVAL;
    }

    if (ret) {
        free(dev_ctx);
        return ret;
    }
    if (m_ctx->sector_size)
        dev_ctx->sector_size = m_ctx->sector_size;
    if (m_ctx->volume_offset)
        dev_ctx->volume_offset = m_ctx->volume_offset;
    *hdev = dev_ctx;
    return 0;
}

rf_int_t win_close(rf_handle_t dev)
{
    rf_disk_context_ref_t dc;
    rf_bool_t ret = FALSE;

    dc = (rf_disk_context_ref_t)dev;
    switch (dc->type) {
    case TYPE_FILE:
        ret = file_drv_close(dev);
        break;

    case TYPE_DISK:
        ret = disk_drv_close(dev);
        break;

    default:
        return RF_EINVAL;
    }

    if (ret == 0)
        free(dc);
    return ret;
}

rf_int_t win_ioctl(rf_handle_t dev, rf_int_t cmd, rf_void_t* buf, rf_int_t len)
{
    rf_disk_context_ref_t dc;

    dc = (rf_disk_context_ref_t)dev;
    switch (dc->type) {
    case TYPE_FILE:
        return file_drv_ioctl(dev, cmd, buf, len);

    case TYPE_DISK:
        return file_drv_ioctl(dev, cmd, buf, len);

    default:
        return RF_EINVAL;
    }
}

rf_int_t win_read(rf_handle_t dev, rf_lloff_t sec_offset, rf_u32_t sec_count, rf_void_t* buf)
{
    rf_disk_context_ref_t dc;

    dc = (rf_disk_context_ref_t)dev;
    switch (dc->type) {
    case TYPE_FILE:
        return file_drv_read(dev, sec_offset, sec_count, buf);

    case TYPE_DISK:
        return disk_drv_read(dev, sec_offset, sec_count, buf);

    default:
        return RF_EINVAL;
    }
}

rf_int_t win_write(rf_handle_t dev, rf_lloff_t sec_offset, rf_u32_t sec_count, rf_void_t* buf)
{
    rf_disk_context_ref_t dc;

    dc = (rf_disk_context_ref_t)dev;
    switch (dc->type) {
    case TYPE_FILE:
        return file_drv_write(dev, sec_offset, sec_count, buf);

    case TYPE_DISK:
        return disk_drv_write(dev, sec_offset, sec_count, buf);

    default:
        return RF_EINVAL;
    }
}
